/*
**
**
** MODULE: client.c
**
**
** PROGRAM: Portable PHNBK Application (OpenVMS,DOS,NT,OSF/1,ULTRIX)
**
**
** ABSTRACT: PHNBK is a sample RPC application intended to illustrate
**           the basics of RPC interoperation between DCE platforms
**           and Microsoft platforms.
**
**          
**
**
*/      

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#ifndef VMS
#include <malloc.h>
#endif
#include "phnbk.h"

#if defined(MSDOS) || defined(WIN32) 
#include "dceport.h" 
#endif

#ifdef WIN32
#define MAIN_DECL _CRTAPI1 
#else
#define MAIN_DECL
#endif


int 
MAIN_DECL main 
              (
               ac, 
               av
              )
int   ac;
char *av[];
{

    int               i;                    
    int               lookup_status;       /* lookup return status */
    error_status_t    status;              /* rpc status */
    idl_char          input[LINESIZE];     /* 'find' search string */
    idl_char          output[LINESIZE];    /* string returned from database*/
    idl_char          oldmatch[LINESIZE];  /* previous 'find' string */
    char              command;             /* lookup command */
    unsigned char     server[80];          /* string binding for server */
  


    /*
    ** Initialize some strings
    */
    oldmatch[0] = '\0';
    server[0]   = '\0';
   
    if (ac < 2)
        {
        /*
        ** With this version of phonbook, the user must specify the 
        **   server's binding as a string on the command line.  If the
        **   user hasn't done that, then tell them about the error and
        **   exit the program so they can try again.
        **
        */
        printf("\nPlease specify a server name on the command line:\n");
        printf("\n\tphnbk myhost\n\n");
      
        exit (EXIT_FAILURE);

        }
    else
        {
        /*
        ** The user has specified a string binding
        */
        strcat ((char *)server,"ncacn_ip_tcp:");
        strcat ((char *)server,av[1]);
        }

    /*
    ** Display the server in use
    */
    printf("\n\t(Selected server binding: %s)\n\n",server);       

    /*
    ** Convert the character string binding into an RPC handle
    */
    rpc_binding_from_string_binding
               (
                server, 
                &xhandle, 
                &status
               );

    if (status)
        {
        printf("Invalid string binding\n");
        exit (EXIT_FAILURE);
        }

    /* 
    ** Usage -- present user with a list of valid commands
    */ 
    printf("Valid commands are:\n");
    printf("(b)rowse        - List next entry\n");
    printf("(r)eset         - Reset to beginning of file\n");
    printf("(f)ind <string> - Find a substring\n");
    printf("(f)ind          - Find next occurrence of <string>\n");
    printf("(q)uit          - Exit program\n\n");
  
    /*
    ** Keep looking for further commands until the user issues the
    **    (q)uit command...
    */
    while(1)
        {
        /*
        ** Initialize input and output strings
        */
        input[0]  = '\0';
        output[0] = '\0';


        /*
        ** Wait for user input
        */
        command = getchar();   


        /*
        ** Select on user input
        */
        switch(command)
            {
            /*
            ** Command "f" means find the given search 
            **    string, or find the next occurrence 
            **    of the previously supplied search string
            */
            case 'f' :
                
                i = 0;
                /*
                ** skip spaces
                */
                while ((command = getchar()) == ' ');
                                 
                /*
                ** Retrieve search string (if there is one)
                */  
                input[i++] = command;
                    
                if (command != '\n')
                    {
                    /*
                    ** There is a search string so retrieve it
                    */
                    while ((input[i++] = getchar()) != '\n');

                    input[--i] = '\0';    

                    strcpy ( (char *)oldmatch, (char *)input );

                    /*
                    ** Try to find a match for the search string
                    **    in the phonebook file
                    */
                    lookup_status = lookup 
                                          (
                                           FIRSTMATCH,
                                           input,
                                           output
                                          );
                    /*
                    ** Print the search results: either a
                    **    match was found or it wasn't...
                    */
                    if (lookup_status == END)
                        printf("No matches found\n");
                    else
                        printf("Entry is: %s\n",output);
                    }

                /*
                ** No search string: search for next occurrence 
                **    of the search string given previously
                */
                else
                    {
                    input[0] = '\0';

                    /*
                    ** restore search string
                    */
                    strcpy ((char *)input, (char *)oldmatch);

                    /*
                    ** search for match
                    */
                    lookup_status = lookup
                                          (
                                           NEXTMATCH,
                                           input,
                                           output
                                          );
                    /*
                    ** Print search results
                    */
                    if (lookup_status == END)
                        printf("No further matches found\n");
                    else
                        printf("Entry is: %s\n",output);
                    }
       
                break;
            /*
            ** Command "b" means just return the next entry
            **    in the phonebook file.
            */
            case 'b' :
     
                /*
                ** Display next entry from wherever we happen to be 
                **    positioned in the phonebook
                */
                lookup_status = lookup
                                      (
                                       BROWSE, 
                                       input, 
                                       output
                                      );

                /*
                ** Echo entry or rewind database if we're at the end
                **    of the phonebook
                */
                if (lookup_status != END)
                    printf("Entry is: %s\n", output);
                else
                    printf("Resetting...\n");

                break;

            /*
            ** Command "r" means reset file position to
            **    the beginning of the phonebook file
            */   
            case 'r' :
     
                input[0] = '\0';

                /*
                ** Reset phonebook position to beginning 
                */
                lookup_status = lookup
                                      (
                                       RESET,
                                       input,
                                       output
                                      );

                printf("Resetting...\n");

                break;
           
            /*
            ** Command "q" means exit the program
            */
            case 'q' :
     
                /*
                ** Free binding handle
                */
                rpc_binding_free
                          (
                           &xhandle,
                           &status
                          );

                /*
                ** Exit program
                */
                printf("Exiting...\n");

                exit(EXIT_SUCCESS);

            default :

                break;

            /*
            ** End of "switch" block
            */  
            }
        
        /*
        ** End of "while" block
        */     
        } 
  

}


#if defined(MSDOS) || defined(WIN32)

/*************************************************************************/
/***                MIDL_user_allocate / MIDL_user_free                ***/
/*************************************************************************/

void * __RPC_API
MIDL_user_allocate
           (
            size
           )
size_t size;
{

    unsigned char * ptr;
    ptr = malloc( size );
    return ( (void *)ptr );
    
}

void __RPC_API
MIDL_user_free
           (
            object
           )
void * object;
{
    free (object);
}

#endif
