/*
**
**
** MODULE: wclient.c
**
**
** PROGRAM: Windows WPHNBK2 Application 
**
**
** ABSTRACT: WPHNBK2 is a sample RPC application intended to illustrate
**           the basics of RPC interoperation between DCE platforms
**           and Microsoft Windows platforms.  WPHNBK2 extends the
**           capabilities of WPHNBK by using context handles, and the
**           CDS nameservice.
**
**          
**
**
*/      

#include <windows.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "phnbk2.h"
#include "wphnbk2.h"

#include "dceport.h"

#define IFSPEC phnbk2_ClientIfHandle
#define SERVER_ENTRY_NAME2 "/.:/phnbk2_srv_"
#define SERVER_ENTRY_NAME  "/.:/subsys/DEC/pc/phnbk2_srv_"


int            lookup_status;     /* lookup return status          */
error_status_t status;            /* rpc status                    */
idl_char       input[LINESIZE];   /* 'find' search string          */
idl_char       output[LINESIZE];  /* string returned from database */
idl_char       oldmatch[LINESIZE];/* previous 'find' string        */
unsigned char  server[80];        /* string binding for server     */
operations     operation;         /* operation requested           */
short          no_handle;         /* handle not initialized flag   */
phonebook_handle ph;              /* phnbk context handle          */

long FAR PASCAL WndProc (HWND, WORD, WORD, LONG) ;

         

int 
PASCAL WinMain 
              (
               HANDLE hInstance, 
               HANDLE hPrevInstance,
               LPSTR lpszCmdLine, 
               int nCmdShow
              )
{
    char szAppName [] = "WPHNBK2" ;
    HWND        hwnd ;
    MSG         msg;
    WNDCLASS    wndclass ;

    /*
    ** Initialize strings
    */
    input[0]    = '\0';
    output[0]   = '\0';
    oldmatch[0] = '\0';
    server[0]   = '\0';

    no_handle = TRUE;

    /*
    ** Standard Windows stuff...
    */
    if (!hPrevInstance)
        {
        wndclass.style          = CS_HREDRAW | CS_VREDRAW;
        wndclass.lpfnWndProc    = WndProc ;
        wndclass.cbClsExtra     = 0 ;
        wndclass.cbWndExtra     = DLGWINDOWEXTRA ;
        wndclass.hInstance      = hInstance ;
        wndclass.hIcon          = LoadIcon(hInstance,szAppName);
        wndclass.hCursor        = LoadCursor (NULL, IDC_ARROW) ;
        wndclass.hbrBackground  = COLOR_WINDOW + 1 ;
        wndclass.lpszMenuName   = NULL ;
        wndclass.lpszClassName  = szAppName ;

        RegisterClass (&wndclass) ;
        }

    hwnd = CreateDialog (hInstance, szAppName, 0, NULL) ;

    ShowWindow (hwnd, nCmdShow) ;

    SetFocus ( GetDlgItem (hwnd, SEARCHBOX ) );

    /*
    ** Start accepting messages
    */
    while ( GetMessage (&msg, NULL, 0, 0) )
        {
        TranslateMessage (&msg) ;
        DispatchMessage (&msg) ;
        }

    return msg.wParam ;
}

void
ErrorBox 
        (
         HWND hwnd,
         unsigned char *error_string
        )
{
    /*
    ** This is just a convenience procedure for
    **    generating message boxes when errors
    **    are encountered.
    */
    MessageBox 
              (
               hwnd,
               error_string,
               "ERROR", 
               MB_OK
              );
}

short
InitHandle
          (
           HWND hwnd
          )
{
    /*
    ** This procedure goes to the CDS nameservice to locate
    **    a PHNBK2 server, initializes the binding handle,
    **    and establishes the context handle.
    */
    unsigned char    *string_binding;
    unsigned char    *protseq;
    handle_t          import_handle;
    long              entry_name_syntax=3;
    rpc_ns_handle_t   import_context;
    int               found=0;
    error_status_t    status;
    error_status_t    status2;
    
 TRY

    /*
    ** Get ready to import bindings that match the
    **    given entry name
    */
    rpc_ns_binding_import_begin
               (
                rpc_c_ns_syntax_default,
                SERVER_ENTRY_NAME,
                IFSPEC,
                NULL,
                &import_context,
                &status
               );

    if (status)
       {
       ErrorBox(hwnd,"import_begin failure");    
       exit (EXIT_FAILURE);
       }

    /*
    ** search for TCP binding
    */
    while (!found)
        {
        /*
        ** Get next binding
        */
        rpc_ns_binding_import_next
                   (
                    import_context,
                    &import_handle,
                    &status
                   );

        if (status)
            { 
            ErrorBox(hwnd,"server not found");    
            exit (EXIT_FAILURE);
            }
        /*
        ** Convert binding to string binding
        */    
        rpc_binding_to_string_binding
                   (
                    import_handle, 
                    &string_binding,
                    &status
                   );

        if (status)
            {
            ErrorBox(hwnd,"binding_to_string_binding failure");    
            exit (EXIT_FAILURE);
            }
        else
            {
            /*
            ** Parse the binding into its individual components...
            **    actually all we care about here is the protocol
            **    sequence...
            */
            rpc_string_binding_parse 
                       (
                        string_binding, 
                        NULL, 
                        &protseq, 
                        NULL, 
                        NULL, 
                        NULL,
                        &status
                       );
                 
         
            if (status)
                {
                ErrorBox(hwnd,"string_binding_parse failure");    
                exit (EXIT_FAILURE);
                }
            else
                {
                /*
                ** Check for a TCP/IP protocol sequence
                */
                if (!(strcmp((char *)protseq, "ncacn_ip_tcp")))
                    /*
                    ** Got one!
                    */ 
                    {
                    strcpy(server, (char *)string_binding);
                    /*
                    ** Free the string binding
                    */                 
                    rpc_string_free
                               (
                                &string_binding, 
                                &status2
                               );
                    found = 1;

                    /*
                    ** finish the search
                    */
                    rpc_ns_binding_import_done(&import_context, &status);
                    }
                else
                    rpc_string_free
                               (
                                &string_binding, 
                                &status2
                               );
                } 

            }
    /*
    ** End of while block
    */
    } 
    rpc_binding_from_string_binding
               (
                server,
                &xhandle,
                &status
               );

    if (status)
        {
        ErrorBox(hwnd,"invalid string binding");
        exit (EXIT_FAILURE);
        }

    /*
    ** Initialize context handle to NULL
    */
    ph = NULL;

    /*
    ** Establish the phonebook context handle
    */
    open_phonebook_file
               (
                &ph
               );

 CATCH_ALL
    /*
    ** Catch all exceptions from above code
    */
    unsigned long exception;
    unsigned char error_code[6];

    exception = RpcExceptionCode();

    ltoa (exception,error_code,10);
   
    ErrorBox (hwnd,strcat("Exception: ",error_code));

    exit (EXIT_FAILURE);


 ENDTRY

    /*
    ** Only way to return successfully
    */   
    
    no_handle = FALSE;

    return (0);

}


void 
ShowResult 
          (
           HWND hwnd
          )
{
    /*
    ** Display lookup results, based on the context of
    **    the requested operation
    */
    if (operation == BROWSE)
        {
        /*
        ** BROWSE -- return next entry
        */
        if (lookup_status == NORMAL)
            /*
            ** Everything ok, display next entry
            */
            SetDlgItemText (hwnd,RESULTSBOX,output);
        else
            {
            /*
            ** Othwise, we hit end of file...
            */
            SetDlgItemText (hwnd,RESULTSBOX,"");
            SetDlgItemText (hwnd,INFOBOX,"No more entries");
            }
        }
    else
        {
        /*
        ** Operation was a Find or Find Next...tailor message
        **    syntax to reflect the operation.
        */
        if (lookup_status == NORMAL)
            {
            /*
            ** Print results
            */
            SetDlgItemText (hwnd,RESULTSBOX,output);
         
            /*
            ** Determine if this was first match, or subsequent match
            */
            if (operation == FIRSTMATCH)
                SetDlgItemText (hwnd,INFOBOX,"Match found");
            else
                SetDlgItemText (hwnd,INFOBOX,"Another match found");
            }
        else
            /*
            ** Hit end of file during search
            */
            if (operation == FIRSTMATCH)
                SetDlgItemText (hwnd,INFOBOX,"Match not found");
            else
                SetDlgItemText (hwnd,INFOBOX,"No other matches found");
        }

    return;
    }


long 
FAR PASCAL WndProc 
                  (
                   HWND hwnd, 
                   WORD message, 
                   WORD wParam, 
                   LONG lParam
                  )
    {
    /*
    ** We switch cursors to the hourglass during
    **    a lookup RPC.  This is for saving the
    **    regular pointer.
    */
    HCURSOR OldCursor;


    /*
    ** First thing, save the match string from last time around
    */
    strcpy (oldmatch, input);

    /*
    ** Switch on the incoming message type (standard Windows
    **    programming)
    */
    switch (message)
        {
        /*
        ** Got a button pushed
        */
        case WM_COMMAND:

            switch (wParam)
                {
                /*
                ** Either a Find or a Find Next
                */
                case FINDBUTTON:

                    /*
                    ** Make sure our handle is initialized
                    */
                    if (no_handle) 
                        if (InitHandle(hwnd)) break;
                    /*
                    ** Clear current text
                    */
                    SetDlgItemText(hwnd,RESULTSBOX,"");
                    SetDlgItemText(hwnd,INFOBOX,"");

                    /*
                    ** Read the search string
                    */
                    GetDlgItemText(hwnd,SEARCHBOX,input,32);

                    /*
                    ** Make sure user entered a search string
                    */
                    if (input[0] == NULL)
                        {
                        ErrorBox (
                                  hwnd,
                                   "Missing Search String!"
                                 );
                        /*
                        ** Set focus back to SEARCHBOX so user can
                        **    enter search string
                        */
                        SetFocus ( GetDlgItem (hwnd, SEARCHBOX) );
                        }
                    else
                        {
                        /*
                        ** Search string is present. Save existing 
                        **    pointer and display hourglass
                        */
                        OldCursor = SetCursor (LoadCursor(NULL,IDC_WAIT));
                        ShowCursor (TRUE);

                        /*
                        ** Determine desired operation
                        */
                        if (strcmp(oldmatch,input))
                            operation = FIRSTMATCH;
                        else
                            operation = NEXTMATCH;

                        /*
                        ** Perform the requested operation
                        */
                        lookup_status = lookup
                                              (
                                               ph,
                                               operation,
                                               input,
                                               output
                                              );

                        /*
                        ** Restore pointer cursor
                        */
                        ShowCursor (FALSE);
                        SetCursor ( OldCursor );
       
                        /*
                        ** Display lookup results
                        */
                        ShowResult(hwnd);
                        }
                    
                    break;

                /*
                ** BROWSE -- return next entry
                */
                case BROWSEBUTTON:

                    /*
                    ** Make sure handle is initialized
                    */
                    if (no_handle) 
                        if(InitHandle(hwnd)) break;
                    /*
                    ** Clear existing text and display status
                    */
                    SetDlgItemText(hwnd,RESULTSBOX,"");
                    SetDlgItemText(hwnd,SEARCHBOX,"");
                    SetDlgItemText(hwnd,INFOBOX,"Browsing...");

                    /*
                    ** Switch to hourglass cursor
                    */
                    OldCursor = SetCursor (LoadCursor(NULL,IDC_WAIT));
                    ShowCursor (TRUE);

                       
                    operation = BROWSE;

                    /*
                    ** Perform the requested operation
                    */
                    lookup_status = lookup 
                                          ( 
                                           ph,
                                           operation, 
                                           input,
                                           output
                                          );

                    /*
                    ** Restore pointer cursor
                    */
                    ShowCursor (FALSE);
                    SetCursor ( OldCursor );
                       
                    /*
                    ** Display operation results
                    */
                    ShowResult(hwnd);
                       
                    break;

                /*
                ** User has requested a RESET.  This clears all
                **    text and rewinds the phonebook file
                */
                case RESETBUTTON:

                    /*
                    ** Make sure handle is initialized
                    */
                    if (no_handle) 
                        if (InitHandle(hwnd)) break;
                    /*
                    ** Clear all text
                    */
                    SetDlgItemText(hwnd,RESULTSBOX,"");
                    SetDlgItemText(hwnd,INFOBOX,"");
                    SetDlgItemText(hwnd,SEARCHBOX,"");

                    input[0] = '\0';

                    operation = RESET;

                    /*
                    ** Perform the requested operation
                    */
                    lookup_status = lookup 
                                          ( 
                                           ph,
                                           operation, 
                                           input,
                                           output
                                          );

                    break;

                }

            return 0 ;

        /*
        ** User has closed the application
        */
        case WM_DESTROY:

            if (!no_handle)
                {
                /*
                ** Free resources associated with context handle
                */
                close_phonebook_file
                           (
                            &ph
                           );

                /*
                ** Free binding handle, post quit message and leave
                */
                rpc_binding_free
                           (
                            &xhandle,
                            &status
                           );
                }

            PostQuitMessage (0) ;
            return 0 ;

        /*
        ** Ignore other messages
        */
        default:
            return DefWindowProc (hwnd, message, wParam, lParam) ;
        }
}
